 
local _
local version = "v0.17c"
local debugmode = false

local ignore_list = {
	[44731] = true, --  Bouquet of Ebon Roses
	[38506] = true, -- Don Carlos' Famous Hat
	[86579] = true, --  Bottled Tornado
	[25653] = true, --   Riding Crop
	[19970] = true, --  Arcanite Fishing Pole
	[118372] = true, -- Orgrimmar Tabard
	[63207] = true, -- Wrap of Unity
	[63353] = true, --Shroud of Cooperation
	[116913] = true, --Peon's Mining Pick
	[116916] = true, --Gorepetal's Gentle Grasp
	[33820] = true, --Weather-Beaten Fishing Hat
	[84661] = true, --Dragon Fishing Pole
	[63206] = true, --Wrap of Unity
	[103678] = true, --Time-Lost Artifact
	[65274] = true, --Cloak of Coordination
	[2901] = true, --Mining Pick
}

local can_transmog = {
	["INVTYPE_HEAD"] = true,
	["INVTYPE_SHOULDER"] = true,
	["INVTYPE_BODY"] = true,
	["INVTYPE_CHEST"] = true,
	["INVTYPE_ROBE"] = true,
	["INVTYPE_WAIST"] = true,
	["INVTYPE_LEGS"] = true,
	["INVTYPE_FEET"] = true,
	["INVTYPE_WRIST"] = true,
	["INVTYPE_HAND"] = true,
	["INVTYPE_CLOAK"] = true,
	["INVTYPE_WEAPON"] = true,
	["INVTYPE_SHIELD"] = true,
	["INVTYPE_2HWEAPON"] = true,
	["INVTYPE_WEAPONMAINHAND"] = true,
	["INVTYPE_WEAPONOFFHAND"] = true,
	["INVTYPE_HOLDABLE"] = true,
	["INVTYPE_RANGED"] = true,
	["INVTYPE_THROWN"] = true,
	["INVTYPE_RANGEDRIGHT"] = true,
	["INVTYPE_RELIC"] = true,
}

local is_weapon_model = {
	["INVTYPE_WEAPON"] = true,
	["INVTYPE_2HWEAPON"] = true,
	["INVTYPE_WEAPONMAINHAND"] = true,
	["INVTYPE_WEAPONOFFHAND"] = true,
}

local config_table = {
	profile = {
		SellGreen = true,
		SellGreenMaxLevel = 460,
		SellBlue = false,
		SellVendorThreshold = 500,
		ReverseSell = false,
		AutoOpenForMerchants = false,
		AutoSellGray = true,
		AllowToSell = {
			["INVTYPE_HEAD"] = true,
			["INVTYPE_NECK"] = true,
			["INVTYPE_SHOULDER"] = true,
			["INVTYPE_BODY"] = true,
			["INVTYPE_CHEST"] = true,
			["INVTYPE_ROBE"] = true,
			["INVTYPE_WAIST"] = true,
			["INVTYPE_LEGS"] = true,
			["INVTYPE_FEET"] = true,
			["INVTYPE_WRIST"] = true,
			["INVTYPE_HAND"] = true,
			["INVTYPE_FINGER"] = true,
			["INVTYPE_TRINKET"] = true,
			["INVTYPE_CLOAK"] = true,
			["INVTYPE_WEAPON"] = true,
			["INVTYPE_SHIELD"] = true,
			["INVTYPE_2HWEAPON"] = true,
			["INVTYPE_WEAPONMAINHAND"] = true,
			["INVTYPE_WEAPONOFFHAND"] = true,
			["INVTYPE_HOLDABLE"] = true,
			["INVTYPE_RANGED"] = true,
			["INVTYPE_THROWN"] = true,
			["INVTYPE_RANGEDRIGHT"] = true,
			["INVTYPE_RELIC"] = true,
		},
	},
}
local options_table = {
	name = "Auto Seller",
	type = "group",
	args = {
	 
		SellGreen = {
			type = "toggle",
			name = "Sell Green Itens",
			desc = "Sell Green Itens.",
			order = 1,
			get = function() return SalvageYardHelper.db.profile.SellGreen end,
			set = function (self, val) 
				SalvageYardHelper.db.profile.SellGreen = not SalvageYardHelper.db.profile.SellGreen; 
				if (SalvageYardHelper.SellPanel) then
					SalvageYardHelper.SellPanel.SellGreenSwitch:SetValue (SalvageYardHelper.db.profile.SellGreen)
				end
			end,
		},
		
		SellBlue = {
			type = "toggle",
			name = "Sell Blue Itens",
			desc = "Sell Blue Itens.",
			order = 2,
			get = function() return SalvageYardHelper.db.profile.SellBlue end,
			set = function (self, val) 
				SalvageYardHelper.db.profile.SellBlue = not SalvageYardHelper.db.profile.SellBlue; 
				if (SalvageYardHelper.SellPanel) then
					SalvageYardHelper.SellPanel.SellBlueSwitch:SetValue (SalvageYardHelper.db.profile.SellBlue)
				end
			end,
		},
	
		SellGreenRange = {
			type = "range",
			name = "Sell Green Level Max", 
			desc = "Itens with item level superior of this, won't be selled.",
			min = 6,
			max = 611,
			step = 1,
			get = function() return SalvageYardHelper.db.profile.SellGreenMaxLevel end,
			set = function (self, val) 
				SalvageYardHelper.db.profile.SellGreenMaxLevel = val; 
				if (SalvageYardHelper.SellPanel) then
					SalvageYardHelper.SellPanel.GreenIlvlMax.value = val 
				end
			end,
			order = 3,
		},
		
		SellByPrice = {
			type = "range",
			name = "Vendor Sell Price Threshold", 
			desc = "Items with a vendor value (in golds) |cFFFFFF00bigger than|r this, will be sold anyway ignoring the item level.",
			min = 15,
			max = 500,
			step = 5,
			get = function() return SalvageYardHelper.db.profile.SellVendorThreshold end,
			set = function (self, val) 
				SalvageYardHelper.db.profile.SellVendorThreshold = val; 
				if (SalvageYardHelper.SellPanel) then
					SalvageYardHelper.SellPanel.GoldThreshold.value = val 
				end
			end,
			order = 5,
		},		
		
		Inverse = {
			type = "toggle",
			name = "Inverse Selling",
			desc = "When inverted, sells itens '|cFFFFFF00higher than|r' instead of '|cFFFFFF00lower than|r'.",
			order = 4,
			get = function() return SalvageYardHelper.db.profile.ReverseSell end,
			set = function (self, val) 
				SalvageYardHelper.db.profile.ReverseSell = not SalvageYardHelper.db.profile.ReverseSell; 
				if (SalvageYardHelper.SellPanel) then
					SalvageYardHelper:CheckReverse() 
				end
			end,
		},
		
		AutoOpenForMerchants = {
			type = "toggle",
			name = "Auto Open",
			desc = "Auto open the window at any merchant.",
			order = 6,
			get = function() return SalvageYardHelper.db.profile.AutoOpenForMerchants end,
			set = function (self, val) 
				SalvageYardHelper.db.profile.AutoOpenForMerchants = val; 
			end,
		},
		
		AutoSellGray = {
			type = "toggle",
			name = "Auto Sell Gray",
			desc = "Auto sell gray items when any merchat window pops up.",
			order = 7,
			get = function() return SalvageYardHelper.db.profile.AutoSellGray end,
			set = function (self, val) 
				SalvageYardHelper.db.profile.AutoSellGray = val; 
			end,
		},

		OpenWindow = {
			type = "execute",
			func = function() 
				SalvageYardHelper:ShowPanel()
			end,
			desc = "Opens the main window.\n\nYou may use at any time /syh to show it.",
			name = "Open Sell Window",
			order = 8,
		},
		
		ConfigBlackList = {
			type = "execute",
			func = function() 
				SalvageYardHelper:ShowPanel()
				SalvageYardHelper.SellPanel.IgnoreButton:Click()
			end,
			desc = "Config the black list to avoid sell some items.\n\nYou may use at any time /syh to popup the sell window.",
			name = "Config Black List",
			order = 9,
		},
	}
}

local broker
local SYH = LibStub ("DetailsFramework-1.0"):CreateAddOn ("SalvageYardHelper", "SalvageYardHDB", config_table, options_table, broker)

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

SYH.GarrisonMaps = {
	["garrisonffhorde_tier1"] = true,
	["garrisonffhorde_tier2"] = true,
	["garrisonffhorde_tier3"] = true,
	["garrisonsmvalliance_tier1"] = true,
	["garrisonsmvalliance_tier2"] = true,
	["garrisonsmvalliance_tier3"] = true,
	["garrisonsmvalliance_tier4"] = true,
}

function SYH:CheckZone()
	SetMapToCurrentZone()
	local mapFileName = GetMapInfo()
	
	if (SYH.GarrisonMaps [mapFileName]) then
		SYH:RegisterEvent ("PLAYER_TARGET_CHANGED", "CheckNpc")
	else
		SYH:UnregisterEvent ("PLAYER_TARGET_CHANGED")
	end
	
	if (debugmode) then
		SYH:ShowPanel()
	end
end

SYH:RegisterEvent ("ZONE_CHANGED_NEW_AREA", "CheckZone")
SYH:RegisterEvent ("PLAYER_ENTERING_WORLD", "CheckZone")

function SYH:CheckNpc()
	local target_guid = UnitGUID ("target")
	if (target_guid) then
		local NpcId = SYH:GetNpcIdFromGuid (target_guid)
		--> 79857 = Lumba the Crusher
		--> 77378 = Hennick Helmsley
		if (NpcId and (NpcId == 79857 or NpcId == 77378)) then
			SYH:ShowPanel()
		end
	end
end

function SYH:ShowPanel (only_load)
	
	if (not SYH.SellPanel) then
	
		SalvageYardHDB.UserBlackList = SalvageYardHDB.UserBlackList or {}
		SalvageYardHDB.UserBlackListKeyWord = SalvageYardHDB.UserBlackListKeyWord or {}
	
		local sell = function (self)
			if (MerchantFrame and MerchantFrame:IsShown()) then
				return SYH:Sell()
			else
				return SYH:Msg ("Merchant Store isn't in your screen. Interact the the Npc.")
			end
		end
		
		local open_ignore = function (self)
			if (not SYH.IgnorePanel) then
			
				SYH.db.profile.IgnorePanel = SYH.db.profile.IgnorePanel or {}
				SYH.IgnorePanel = SYH:Create1PxPanel (_, 255, 120, "Ignore Items", "SalvageYardIgnoreFrame", SYH.db.profile.IgnorePanel)
				SYH.IgnorePanel:SetPoint ("topleft", SalvageYardHelperFrame, "topright", 5, 0)
				
				local text = SYH:CreateLabel (SYH.IgnorePanel, "Enter the Item Name:")
				local editbox = SYH:CreateTextEntry (SYH.IgnorePanel, _, 160, 20, "EditBox")
				editbox.tooltip = "You may link the item directly from your backpack using |cFFFFFF00SHIFT|r key."
				editbox.latest_item = {}
				
				-- Hook the container buttons
				local hook_backpack = function (button)
					if (editbox:HasFocus()) then
						if (IsShiftKeyDown()) then
							local link = GetContainerItemLink (button:GetParent():GetID(), button:GetID())
							local name = GetItemInfo (link)
							editbox.text = name or ""
							editbox.latest_item [1] = name
							editbox.latest_item [2] = link
						end
					end
				end
				hooksecurefunc ("ContainerFrameItemButton_OnModifiedClick", hook_backpack)
				
				local add_to_ignore = function()
					local text = editbox.text
					text = SYH:trim (text)
					if (text ~= "") then
					
						local itemname, itemlink
						
						if (editbox.text == editbox.latest_item [1]) then
							itemname = editbox.latest_item [1]
							itemlink = editbox.latest_item [2]
						else
							text = text:gsub ("%[", "")
							text = text:gsub ("%]", "")
							itemname = text
						end
						
						if (itemname and itemlink) then
							print (itemname, itemlink)
							SalvageYardHDB.UserBlackList [itemname] = itemlink or true
							SYH:Msg (itemname .. " added to ignore list.")
						else
							SalvageYardHDB.UserBlackList [text] = true
							SYH:Msg (text .. " added to ignore list.")
						end
						
						editbox.text = ""
						editbox:ClearFocus()
					end
				end
				local add_button = SYH:CreateButton (SYH.IgnorePanel, add_to_ignore, 45, 20, "Add", _, _, _, "AddButton", _, _)
			
				text:SetPoint ("topleft", SYH.IgnorePanel, "topleft", 7, -30)
				editbox:SetPoint ("topleft", SYH.IgnorePanel, "topleft", 7, -42)
				add_button:SetPoint ("left", editbox, "right", 4, 0)

				local remove_ignore = SYH:CreateButton (SYH.IgnorePanel, function()end, 6, 20, "X", _, _, _, "RemoveButton", _, _)
				remove_ignore:SetPoint ("left", add_button, "right", 2)
				local remove_func = function (_, _, itemname)
					SalvageYardHDB.UserBlackList [itemname] = nil
					GameCooltip2:Hide()
					SYH:Msg (itemname .. " removed from item list.")
				end
				remove_ignore:SetHook ("OnEnter", function()
					GameCooltip2:Preset (2)
					
					for itemname, _ in pairs (SalvageYardHDB.UserBlackList) do
						GameCooltip2:AddLine (itemname)
						GameCooltip2:AddMenu (1, remove_func, itemname)
					end

					GameCooltip2:SetType ("menu")
					GameCooltip2:SetHost (remove_ignore.widget, "left", "right", -7, 0)
					GameCooltip2:Show()
				end)
				
				-----
				
				local text2 = SYH:CreateLabel (SYH.IgnorePanel, "Enter a keyword:")
				local editbox2 = SYH:CreateTextEntry (SYH.IgnorePanel, _, 160, 20, "EditBoxKeyWord")
				editbox2.tooltip = "Items containing the word on its name won't be sold."

				local add_keyword_to_ignore = function()
					local text = editbox2.text
					text = SYH:trim (text)
					text = text:gsub ("%[", "")
					text = text:gsub ("%]", "")
					
					if (text ~= "") then
						text = string.lower (text)
						SalvageYardHDB.UserBlackListKeyWord [text] = true
						SYH:Msg (text .. " added to keyword ignore list.")
						editbox2.text = ""
						editbox2:ClearFocus()
					end
				end
				
				local add_button2 = SYH:CreateButton (SYH.IgnorePanel, add_keyword_to_ignore, 45, 20, "Add", _, _, _, "AddButtonKeyWord", _, _)
			
				text2:SetPoint ("topleft", SYH.IgnorePanel, "topleft", 7, -70)
				editbox2:SetPoint ("topleft", SYH.IgnorePanel, "topleft", 7, -82)
				add_button2:SetPoint ("left", editbox2, "right", 4, 0)
				
				local remove = SYH:CreateButton (SYH.IgnorePanel, function()end, 6, 20, "X", _, _, _, "RemoveButtonKeyWord", _, _)
				remove:SetPoint ("left", add_button2, "right", 2)
				local remove_keyword_func = function (_, _, keyword)
					SalvageYardHDB.UserBlackListKeyWord [keyword] = nil
					GameCooltip2:Hide()
					SYH:Msg (keyword .. " removed from keyword list.")
				end
				remove:SetHook ("OnEnter", function()
					GameCooltip2:Preset (2)
					
					for keyword, _ in pairs (SalvageYardHDB.UserBlackListKeyWord) do
						GameCooltip2:AddLine (keyword)
						GameCooltip2:AddMenu (1, remove_keyword_func, keyword)
					end

					GameCooltip2:SetType ("menu")
					GameCooltip2:SetHost (remove.widget, "left", "right", -7, 0)
					GameCooltip2:Show()
				end)
			end
			
			if (SYH.AdvancedPanel and SYH.AdvancedPanel:IsShown())then
				SYH.AdvancedPanel:Hide()
			end
			
			SYH.IgnorePanel:Show()
		end
		
		local open_advanced = function()
			
			if (not SYH.AdvancedPanel) then
			
				SYH.db.profile.AdvancedPanel = SYH.db.profile.AdvancedPanel or {}
				SYH.AdvancedPanel = SYH:Create1PxPanel (_, 255, 120, "Advanced Options", "SalvageYardAdvancedFrame", SYH.db.profile.AdvancedPanel)
				SYH.AdvancedPanel:SetPoint ("topleft", SalvageYardHelperFrame, "topright", 5, 0)
				
				local sell_soulbound_switch, sell_soulbound_label = SYH:CreateSwitch (SYH.AdvancedPanel, nil, SYH.SellPanel.ForceSellSoulbound, _, _, _, _, "SellSoulboundSwitch", _, _, _, _, "Sell Soulbound:")
				local sell_epic_switch, sell_epic_label = SYH:CreateSwitch (SYH.AdvancedPanel, nil, SYH.SellPanel.SellLowLevelEpic, _, _, _, _, "SellEpicSwitch", _, _, _, _, "Sell Low Level Epic:")
				
				sell_soulbound_switch.tooltip = "Sell soulbound items.\n\n|cFFFFFF00Important:|r make sure to remove important items from your backpack and also perform a sell test to see if any important item are sold.\n\n|cFFFFFF00Important:|r When you logoff, this option back to its default value (disabled)."
				sell_epic_switch.tooltip = "Sell soulbound epic items with item level lower than 600.\n\n|cFFFFFF00Important:|r When you logoff, this option back to its default value (disabled).\n\n|cFFFFFF00Important:|r Make a sell test before a real use and see if any important items are sold."
				
				sell_soulbound_label:SetPoint ("topleft", SYH.AdvancedPanel, "topleft", 7, -40)
				sell_epic_label:SetPoint ("topleft", SYH.AdvancedPanel, "topleft", 7, -60)
				
				sell_soulbound_switch.OnSwitch = function (_, _, value) 
					SYH.SellPanel.ForceSellSoulbound = not SYH.SellPanel.ForceSellSoulbound
				end
				sell_epic_switch.OnSwitch = function (_, _, value) 
					SYH.SellPanel.SellLowLevelEpic = not SYH.SellPanel.SellLowLevelEpic
				end
				
			end
			
			if (SYH.IgnorePanel and SYH.IgnorePanel:IsShown())then
				SYH.IgnorePanel:Hide()
			end
			
			SYH.AdvancedPanel:Show()
			
		end
		
		local open_transmog = function()
			if (not SYH.TransmogPanel) then
				SYH.db.profile.TransmogPanel = SYH.db.profile.TransmogPanel or {}
				SYH.TransmogPanel = SYH:Create1PxPanel (_, 420, 200, "Transmog Panel", "SalvageYardTransmogFrame", SYH.db.profile.TransmogPanel)
				SYH.TransmogPanel:SetPoint ("topleft", SalvageYardHelperFrame, "topright", 5, 0)
				
				SYH.TransmogPanel:SetScript ("OnShow", function()
					SYH.TransmogPanel:RegisterEvent ("BAG_UPDATE")
					SYH.TransmogPanel:UpdateFrames()
				end)
				SYH.TransmogPanel:SetScript ("OnHide", function()
					SYH.TransmogPanel:UnregisterEvent ("BAG_UPDATE")
				end)
				SYH.TransmogPanel:SetScript ("OnEvent", function (self, event, ...)
					if (event == "BAG_UPDATE") then
						SYH.TransmogPanel:UpdateFrames()
					end
				end)
				
				local frame3D = CreateFrame ("DressUpModel", "SalvageYard3DTransmogFrame", 	SYH.TransmogPanel, "ModelWithControlsTemplate")
				frame3D:SetSize (260, 300)
				frame3D:SetFrameStrata ("TOOLTIP")
				frame3D:SetBackdrop ({bgFile = [[Interface\Tooltips\UI-Tooltip-Background]], tileSize = 64, tile = true})
				frame3D:SetBackdropColor (0, 0, 0, 1)
				frame3D.defaultRotation = MODELFRAME_DEFAULT_ROTATION
				frame3D.cloakRotation = MODELFRAME_DEFAULT_ROTATION + math.pi
				frame3D.weaponRotation = MODELFRAME_DEFAULT_ROTATION + (math.pi/5)
				frame3D.bowRotation = math.pi*2*0.75
				frame3D.minZoom = 0
				frame3D.maxZoom = 1
	
				local on_enter = function (self)
					frame3D:Show()
					frame3D:SetPoint ("bottom", self, "top", 0, 5)
					
					Model_Reset (frame3D)
					frame3D:SetUnit ("player")
					frame3D:Undress()
					frame3D:TryOn (self.itemLink)
					
					if (self.isCloak) then
						frame3D:SetRotation (frame3D.cloakRotation)
					elseif (self.isWeapon) then
						frame3D:SetRotation (frame3D.weaponRotation)
					elseif (self.isBow) then
						frame3D:SetRotation (frame3D.bowRotation)
					end
					
					self.texture:SetBlendMode ("ADD")
					
					GameTooltip:SetOwner (self)
					GameTooltip:SetHyperlink (self.itemLink)
					GameTooltip:Show()
					GameTooltip:ClearAllPoints()
					GameTooltip:SetPoint ("topleft", frame3D, "topright", 2, 0)
				end
				local on_leave = function (self)
					frame3D:Hide()
					GameTooltip:Hide()
					self.texture:SetBlendMode ("BLEND")
				end
				
				local on_mouse_up = function (self, button)
					if (button == "LeftButton") then
						if (MerchantFrame and MerchantFrame:IsShown()) then
							if (self.itemQuality >= 4) then
								SYH:Msg ("Cannot sell epic items through this panel.")
							else
								UseContainerItem (self.backPackNumber, self.backPackSlot)
								SYH:Msg ("Total Sold: " .. GetCoinText (self.sellPrice) .. ".")
							end
						end
					end
				end

				SYH.TransmogPanel.blocks = {}
				local x, y = 5, -25
				local backdrop = {edgeFile = [[Interface\Buttons\WHITE8X8]], edgeSize = 1, bgFile = [[Interface\Tooltips\UI-Tooltip-Background]], tileSize = 20, tile = true}
				
				function SYH.TransmogPanel:ClearBlocks()
					for _, block in ipairs (SYH.TransmogPanel.blocks) do --dos
						block:Hide()
					end
				end
				
				function SYH.TransmogPanel:GetBlock (i)
					if (not SYH.TransmogPanel.blocks [i]) then
						local f = CreateFrame ("frame", nil, SYH.TransmogPanel)
						f:SetPoint ("topleft", SYH.TransmogPanel, "topleft", x, y)
						f:SetSize (30, 30)
						f.texture = f:CreateTexture (nil, overlay)
						f.texture:SetPoint ("topleft", f, "topleft", 1, -1)
						f.texture:SetPoint ("bottomright", f, "bottomright", -1, 1)
						f:SetBackdrop (backdrop)
						f:SetScript ("OnEnter", on_enter)
						f:SetScript ("OnLeave", on_leave)
						f:SetScript ("OnMouseUp", on_mouse_up)
						
						if (x > 370) then
							x = 5
							y = y - 31
						else
							x = x + 31
						end
						
						SYH.TransmogPanel.blocks [i] = f
					end
					return SYH.TransmogPanel.blocks [i]
				end
				
				function SYH.TransmogPanel:UpdateFrames()
				
					local i = 1
					SYH.TransmogPanel:ClearBlocks()
				
					for backpack = 0, 4 do
						for slot = 1, GetContainerNumSlots (backpack) do 
							local itemId = GetContainerItemID (backpack, slot)
							if (itemId and not ignore_list [itemId] and not SYH:IsSoulbound (backpack, slot)) then
								local _, _, _, quality, _, _, itemLink = GetContainerItemInfo (backpack, slot)
								local itemName, itemLink, _, itemLevel, _, itemType, itemSubType, _, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo (itemLink)
								if (itemName and quality ~= 0 and quality <= 3 and SYH.db.profile.AllowToSell [itemEquipLoc] and can_transmog [itemEquipLoc] and not SalvageYardHDB.UserBlackList [itemName]) then
									local block = SYH.TransmogPanel:GetBlock (i)
									block.texture:SetTexture (itemTexture)
									block.texture:SetTexCoord (5/64, 59/64, 5/64, 59/64)
									local color = BAG_ITEM_QUALITY_COLORS [quality]
									if (color) then
										block:SetBackdropBorderColor (color.r, color.g, color.b)
									else
										block:SetBackdropBorderColor (.5, .5, .5)
									end
									block.itemLink = itemLink
									block.isCloak = itemEquipLoc == "INVTYPE_CLOAK"
									block.isWeapon = is_weapon_model [itemEquipLoc]
									block.isBow = itemEquipLoc == "INVTYPE_RANGED" or itemEquipLoc == "INVTYPE_SHIELD"
									block.backPackNumber = backpack
									block.backPackSlot = slot
									block.sellPrice = itemSellPrice
									block.itemQuality = quality
									block:Show()
									i = i + 1
								end
							end
						end
					end
				end
				
			end
			
			if (SYH.TransmogPanel and SYH.TransmogPanel:IsShown())then
				SYH.TransmogPanel:Hide()
			else
				SYH.TransmogPanel:Show()
			end
		end
	
		SYH.db.profile.MainPanel = SYH.db.profile.MainPanel or {}
		SYH.SellPanel = SYH:Create1PxPanel (_, 225, 220, "Auto Seller", "SalvageYardHelperFrame", SYH.db.profile.MainPanel)
		
		SYH.SellPanel.ForceSellSoulbound = false
		SYH.SellPanel.SellLowLevelEpic = false
		
--		local f=CreateFrame("frame",nil,UIParent);f:SetSize(200,200);f:SetPoint("center",UIParent,"center");f.t=f:CreateTexture(nil,"overlay");f.t:SetAllPoints();f.t:SetTexture ([[Interface\AddOns\SYH\libs\DF\feedback_sites]])
		
		local feedback_func = function()
			local feedback1 = {icon = [[Interface\AddOns\SYH\libs\DF\feedback_sites]], coords = SYH.www_icons.mmoc, desc = "Post on our thread on MMO-Champion Forum.", link = [[http://www.mmo-champion.com/threads/1755513-AddOn-Salvage-Yard-Seller-(sells-gray-green-blue-items-by-item-level)]]}
			local feedback2 = {icon = [[Interface\AddOns\SYH\libs\DF\feedback_sites]], coords = SYH.www_icons.curse, desc = "Leave a comment on our page at Curse.com.", link = [[http://www.curse.com/addons/wow/salvage-yard-seller]]}
			
			local same1 = {name = "Details! Damage Meter", desc = "A Damage Meter with a lot of tools for raiders and leaders.", link = [[http://www.curse.com/addons/wow/details]], icon = [[Interface\AddOns\SYH\libs\DF\all_addons]], coords = {0, 128/512, 0, 64/512}}
			local same2 = {name = "Gold Token Price", desc = "Adds the slash command '/gold'. This command tells you the current price of WoW Token.", link = [[http://www.curse.com/addons/wow/gold-token-price]], icon = [[Interface\AddOns\SYH\libs\DF\all_addons]], coords = {128/512, 256/512, 0, 64/512}}
			local same3 = {name = "Hansgar & Franzok Assist", desc = "Tool for Hans and Franz encounter on Blackrock Foundry raid.", link = [[http://www.curse.com/addons/wow/hansgar_and_franzok_assist]], icon = [[Interface\AddOns\SYH\libs\DF\all_addons]], coords = {256/512, 384/512, 0, 64/512}}
			local same4 = {name = "AddOns CPU Usage", desc = "Measure the CPU usage by addons. Important to get rid of FPS drops during boss encounters.", link = [[http://www.curse.com/addons/wow/addons-cpu-usage]], icon = [[Interface\AddOns\SYH\libs\DF\all_addons]], coords = {384/512, 512/512, 0, 64/512}}
			local same5 = {name = "Keep World Map Zoom", desc = "Because it's a pain having to re-zoom the world map after close and reopening in a short period of time.", link = [[http://www.curse.com/addons/wow/world-map-zoom]], icon = [[Interface\AddOns\SYH\libs\DF\all_addons]], coords = {0/512, 128/512, 64/512, 128/512}}
			local same6 = {name = "PvPScan", desc = "Show a unit frame with enemy players near you.", link = [[http://www.wowace.com/addons/pvpscan/]], icon = [[Interface\AddOns\SYH\libs\DF\all_addons]], coords = {128/512, 256/512, 64/512, 128/512}}
			local same7 = {name = "HotCorners", desc = "Show a hotcorner when poiting the mouse on the absolute top left of your screen (similar of those on Windows 8).", link = [[http://www.curse.com/addons/wow/hotcorners]], icon = [[Interface\AddOns\SYH\libs\DF\all_addons]], coords = {256/512, 384/512, 64/512, 128/512}}
			
			SYH:ShowFeedbackPanel ("Auto Seller", version, {feedback2, feedback1}, {same1, same2, same3, same4, same5, same6, same7})
		end
		local feedback_button = SYH:CreateFeedbackButton (SYH.SellPanel, feedback_func, "SalvageYardHelperFBButton")
		feedback_button:SetPoint ("right", SYH.SellPanel.Lock, "left", 0, -1)
		
		if (debugmode) then
			feedback_func()
		end
		
		local green_switch, green_label = SYH:CreateSwitch (SYH.SellPanel, nil, SYH.db.profile.SellGreen, _, _, _, _, "SellGreenSwitch", _, _, _, _, "Sell Green:")
		local blue_switch, blue_label = SYH:CreateSwitch (SYH.SellPanel, nil, SYH.db.profile.SellBlue, _, _, _, _, "SellBlueSwitch", _, _, _, _, "Sell Blue:")
		local green_ilvl_slider, green_ilvl_label = SYH:CreateSlider (SYH.SellPanel, _, _, 6, 611, 1, SYH.db.profile.SellGreenMaxLevel, _, "GreenIlvlMax", _, "Item Level:")
		local gold_threshold_slider, gold_threshold_label = SYH:CreateSlider (SYH.SellPanel, _, _, 15, 500, 5, SYH.db.profile.SellVendorThreshold, _, "GoldThreshold", _, "Vendor Gold:")
		local sell_items = SYH:CreateButton (SYH.SellPanel, sell, 120, 20, "Sell Items", _, _, _, "SellButton", _, _)
		local ignore_button = SYH:CreateButton (SYH.SellPanel, open_ignore, 75, 14, "Ignore List", _, _, _, "IgnoreButton", _, _)
		ignore_button:SetBackdropBorderColor (0, 0, 0, 0)
		ignore_button.tooltip = function()
			local s = "Add Items to Black List.\n\nAlready Ignored:\n"
			for itemname, itemlink in pairs (SalvageYardHDB.UserBlackList) do
				if (type (itemlink) == "boolean") then
					s = s .. " -" .. itemname .. "\n"
				else
					s = s .. " -" .. itemlink .. "\n"
				end
			end
			
			s = s .. "\n\nKey Words:\n"
			for keyword, _ in pairs (SalvageYardHDB.UserBlackListKeyWord) do
				s = s .. " \"|cFFFFFFFF" .. keyword .. "|r\" "
			end
			
			return s
		end
		local auto_sell_gray, auto_sell_gray_label = SYH:CreateSwitch (SYH.SellPanel, nil, SYH.db.profile.AutoSellGray, _, _, _, _, "AutoSellGraySwitch", _, _, _, _, "Auto Sell Gray:")
		local auto_open, auto_open_label = SYH:CreateSwitch (SYH.SellPanel, nil, SYH.db.profile.AutoOpenForMerchants, _, _, _, _, "AutoOpenForMerchantsSwitch", _, _, _, _, "Auto Open:")

		local advanced_button = SYH:CreateButton (SYH.SellPanel, open_advanced, 75, 14, "Advanced", _, _, _, "AdvancedButton", _, _)
		advanced_button:SetBackdropBorderColor (0, 0, 0, 0)
		advanced_button.tooltip = function()
			return "Be careful using these advanced features."
		end		
		
		local transmog_button = SYH:CreateButton (SYH.SellPanel, open_transmog, 75, 14, "Transmog", _, _, _, "TransogButton", _, _)
		transmog_button:SetBackdropBorderColor (0, 0, 0, 0)
		
		
		green_switch.tooltip = "Sell green items when enabled."
		blue_switch.tooltip = "Sell blue items when enabled."
		gold_threshold_slider.tooltip = "Items with a vendor value (in golds) |cFFFFFF00bigger than|r this, will be sold anyway ignoring the item level."
		auto_sell_gray.tooltip = "Auto sell gray items when any merchat window pops up."
		auto_open.tooltip = "Auto open the window on any merchant."
		
		green_switch.OnSwitch = function (_, _, value) SYH.db.profile.SellGreen = value end
		blue_switch.OnSwitch = function (_, _, value) SYH.db.profile.SellBlue = value end
		auto_sell_gray.OnSwitch = function (_, _, value) SYH.db.profile.AutoSellGray = value end
		auto_open.OnSwitch = function (_, _, value) SYH.db.profile.AutoOpenForMerchants = value end
		
		local sell_texture_green = green_ilvl_slider:CreateTexture (nil, "background")
		local sell_texture_red = green_ilvl_slider:CreateTexture (nil, "background")
		
		sell_texture_green:SetTexture (.2, 1, .2, 0.2)
		sell_texture_red:SetTexture (1, .2, .2, 0.2)
		sell_texture_green:SetHeight (11)
		sell_texture_red:SetHeight (11)
		green_ilvl_slider.thumb:SetAlpha (1)
		
		green_ilvl_slider.OnValueChanged = function (_, _, value) 
			SYH.db.profile.SellGreenMaxLevel = value 
			SYH:CheckReverse()
		end
		
		gold_threshold_slider.OnValueChanged = function (_, _, value) 
			SYH.db.profile.SellVendorThreshold = value
		end
	
		function SYH:CheckReverse()
			if (SYH.db.profile.ReverseSell) then 
				--> inverted
				sell_texture_red:SetPoint ("left", green_ilvl_slider.thumb, "center")
				sell_texture_red:SetPoint ("right", green_ilvl_slider.widget, "right", -3, 0)
				
				sell_texture_green:SetPoint ("left", green_ilvl_slider.widget, "left", 3, 0)
				sell_texture_green:SetPoint ("right", green_ilvl_slider.thumb, "center")
				
				green_ilvl_slider.tooltip = "Items with item level |cFFFFFF00higher than|r this, will be sold."
				
				SYH.SellPanel.ReverseButton.widget:GetNormalTexture():SetVertexColor (1, 0, 0, 1)
				SYH.SellPanel.ReverseButton.widget:GetPushedTexture():SetVertexColor (1, 0, 0, 1)
				SYH.SellPanel.ReverseButton.widget:GetDisabledTexture():SetVertexColor (1, 0, 0, 1)
				SYH.SellPanel.ReverseButton.widget:GetHighlightTexture():SetVertexColor (1, 0, 0, 1)
			else 
				--> normal
				sell_texture_red:SetPoint ("left", green_ilvl_slider.widget, "left", 3, 0)
				sell_texture_red:SetPoint ("right", green_ilvl_slider.thumb, "center")
				
				sell_texture_green:SetPoint ("left", green_ilvl_slider.thumb, "center")
				sell_texture_green:SetPoint ("right", green_ilvl_slider.widget, "right", -3, 0)
				
				green_ilvl_slider.tooltip = "Items with item level |cFFFFFF00lower than|r this, will be sold."
				
				SYH.SellPanel.ReverseButton.widget:GetNormalTexture():SetVertexColor (1, 1, 1, 1)
				SYH.SellPanel.ReverseButton.widget:GetPushedTexture():SetVertexColor (1, 1, 1, 1)
				SYH.SellPanel.ReverseButton.widget:GetDisabledTexture():SetVertexColor (1, 1, 1, 1)
				SYH.SellPanel.ReverseButton.widget:GetHighlightTexture():SetVertexColor (1, 1, 1, 1)
			end
		end
		
		local do_reverse = function()
			SYH.db.profile.ReverseSell = not SYH.db.profile.ReverseSell
			SYH:CheckReverse()
		end
		local reverse = SYH:CreateButton (SYH.SellPanel, do_reverse, 14, 14, "", _, _, [[Interface\Buttons\UI-RefreshButton]], "ReverseButton", _, _)
		reverse.tooltip = "When inverted, sells itens '|cFFFFFF00higher than|r' instead of '|cFFFFFF00lower than|r'."
		
		SYH:CheckReverse()
		
		green_label:SetPoint ("topleft", SYH.SellPanel, "topleft", 7, -40)
		blue_label:SetPoint ("topleft", SYH.SellPanel, "topleft", 7, -60)
		green_ilvl_label:SetPoint ("topleft", SYH.SellPanel, "topleft", 7, -80)
		gold_threshold_label:SetPoint ("topleft", SYH.SellPanel, "topleft", 7, -100)
		
		auto_sell_gray_label:SetPoint ("topleft", SYH.SellPanel, "topleft", 7, -120)
		auto_open_label:SetPoint ("topleft", SYH.SellPanel, "topleft", 7, -140)
		
		sell_items:SetPoint ("center", SYH.SellPanel, "center", 0, 0)
		sell_items:SetPoint ("top", SYH.SellPanel, "top", 0, -185)
		reverse:SetPoint ("left", green_ilvl_slider, "right", 5, 0)
		ignore_button:SetPoint ("left", green_switch, "right", 10, 0)
		advanced_button:SetPoint ("left", blue_switch, "right", 10, 0)
		transmog_button:SetPoint ("left", auto_open, "right", 10, 0)
		
		local progress_bar = SYH:CreateBar (SYH.SellPanel, _, 210, 10, 0, "ProgressBar")
		progress_bar.shown = false
		progress_bar:SetPoint ("center", SYH.SellPanel, "center", 0, 0)
		progress_bar:SetPoint ("top", SYH.SellPanel, "top", 0, -163)
		
		local tooltip_scanner = CreateFrame ("gametooltip", "SalvageYardHelperGTScanner", nil, "GameTooltipTemplate")
		tooltip_scanner:Hide()
		
		function SYH:IsSoulbound (bag, slot)
			tooltip_scanner:SetOwner (UIParent, "ANCHOR_NONE")
			tooltip_scanner:ClearLines()
			tooltip_scanner:SetBagItem (bag, slot)
			for i = 1, 3 do
				local text = _G ["SalvageYardHelperGTScannerTextLeft" .. i] and _G ["SalvageYardHelperGTScannerTextLeft" .. i]:GetText()
				if (text and text == ITEM_SOULBOUND) then
					return true
				end
			end
			tooltip_scanner:Hide()
		end
		 
		SYH.SellPanel:SetScript ("OnHide", function()
			if (SalvageYardIgnoreFrame) then
				SalvageYardIgnoreFrame:Hide()
			end
			if (SalvageYardTransmogFrame) then
				SalvageYardTransmogFrame:Hide()
			end
			if (SalvageYardAdvancedFrame) then
				SalvageYardAdvancedFrame:Hide()
			end
		end)

	end
	
	SYH.SellPanel:Show()

	if (only_load) then
		SYH.SellPanel:Hide()
	end
	
end

MerchantFrame:HookScript ("OnShow", function (self)
	if (SYH.db.profile.AutoOpenForMerchants) then
		SYH:ShowPanel()
	end
	if (SYH.db.profile.AutoSellGray) then
		SYH:Sell (true)
	end
	if (not self.OpenSYHButton) then
		self.OpenSYHButton = CreateFrame ("button", "SalvageYardHelperOpenButton", self, "OptionsButtonTemplate")
		self.OpenSYHButton:SetSize (40, 19)
		self.OpenSYHButton:SetText ("A.S.")
		--self.OpenSYHButton:SetPoint ("bottomleft", self, "bottomleft", 7, 5)
		--self.OpenSYHButton:SetPoint ("topleft", self, "topleft", 60, -32)
		self.OpenSYHButton:SetPoint ("topright", self, "topright", -26, -1)
		self.OpenSYHButton:SetScript ("OnClick", function (self)
			SYH:ShowPanel()
		end)
	end
end)
MerchantFrame:HookScript ("OnHide", function (self)
	if (SYH.SellPanel) then
		SYH.SellPanel:Hide()
	end
end)

function SYH:Sell (only_gray)

	if (not SYH.SellPanel) then
		SYH:ShowPanel (true)
	end

	local gray_item = 0
	local green_item = 2
	local blue_item = 3
	local epic_item = 4
	
	local sell_green = SYH.db.profile.SellGreen
	local sell_blue = SYH.db.profile.SellBlue
	
	if (only_gray) then
		sell_green = false
		sell_blue = false
	end
	--
	local to_sell = {}
	--
	local GetContainerNumSlots, GetContainerItemInfo, GetItemInfo, UseContainerItem = GetContainerNumSlots, GetContainerItemInfo, GetItemInfo, UseContainerItem
	--
	for backpack = 0, 4 do
		for slot = 1, GetContainerNumSlots (backpack) do 
		
			local itemId = GetContainerItemID (backpack, slot)
			if (itemId and not ignore_list [itemId]) then
				local _, _, _, quality, _, _, itemLink = GetContainerItemInfo (backpack, slot)
				local itemName, itemLink, _, itemLevel, _, itemType, itemSubType, _, itemEquipLoc, _, itemSellPrice = GetItemInfo (itemLink)

				--> gray
				if (quality == gray_item) then
					local valor = itemSellPrice or 0
					to_sell [#to_sell+1] = {backpack, slot, valor}
				
				--> green and blue
				else
					if (itemName and SYH.db.profile.AllowToSell [itemEquipLoc] and not SalvageYardHDB.UserBlackList [itemName]) then
					
						local keyword_free = true
						local low_itemName = string.lower (itemName)
						for keyword, _ in pairs (SalvageYardHDB.UserBlackListKeyWord) do
							if (low_itemName:find (keyword)) then
								keyword_free = false
								break
							end
						end
						
						if (keyword_free) then
							--> green blue
							if ((quality == green_item and sell_green) or (quality == blue_item and sell_blue)) then
								if (SYH.SellPanel.ForceSellSoulbound or not SYH:IsSoulbound (backpack, slot)) then
									if (itemLevel > 5) then
										local _, _, _, _, _, _, _, _, _, _, _, upgradeTypeID = strsplit (":", itemLink)
										if (upgradeTypeID ~= "512") then --> isn't timewarped
											if (not SYH.db.profile.ReverseSell) then
												local valor = itemSellPrice
												if ((itemLevel < SYH.db.profile.SellGreenMaxLevel) or (valor > (SYH.db.profile.SellVendorThreshold * 10000))) then
													to_sell [#to_sell+1] = {backpack, slot, valor}
												end
											else	
												local valor = itemSellPrice
												if ((itemLevel > SYH.db.profile.SellGreenMaxLevel) or (valor > (SYH.db.profile.SellVendorThreshold * 10000))) then
													to_sell [#to_sell+1] = {backpack, slot, valor}
												end
											end
										end
									end
								end
							--> epic
							elseif (quality == epic_item and SYH.SellPanel.SellLowLevelEpic) then
								if (itemLevel < 600 and itemLevel > 5 and SYH:IsSoulbound (backpack, slot)) then
									local valor = itemSellPrice
									to_sell [#to_sell+1] = {backpack, slot, valor}
								end
							end
						end
					end
				end
			end
		end
	end
	
	SYH.SellGoldAmount = 0
	SYH.SellPanel.ProgressBar.value = 0
	SYH.TotalItems = #to_sell
	SYH.SellPanel.ProgressBar.shown = true
	
	if (SYH.SellThreadExec) then
		SYH:CancelTimer (SYH.SellThreadExec)
	end
	
	SYH.SellThreadExec = SYH:ScheduleRepeatingTimer ("SellThread", 0.2, to_sell)
end

function SYH:SellThread (item_list)
	local item = tremove (item_list)
	
	if (not item) then
		return SYH:SellingFinished()
	end
	
	UseContainerItem (item[1], item[2])
	SYH.SellGoldAmount = SYH.SellGoldAmount + item[3]
	
	SYH.SellPanel.ProgressBar.value = abs (#item_list-SYH.TotalItems) / SYH.TotalItems * 100
	
	if (#item_list == 0) then
		SYH:SellingFinished()
	end
end

function SYH:SellingFinished()
	if (type (SYH.SellGoldAmount) == "number" and SYH.SellGoldAmount > 0) then
		SYH:Msg ("Total Sold: " .. GetCoinText (SYH.SellGoldAmount) .. ".")
	end
	SYH.SellPanel.ProgressBar.shown = false
	SYH:CancelTimer (SYH.SellThreadExec)
	SYH.SellThreadExec = nil
end

SLASH_SYH1 = "/syh"
function SlashCmdList.SYH (msg, editbox)
	local command, rest = msg:match ("^(%S*)%s*(.-)$")

	SYH:Msg ("|cFF00FF00" ..version .. "|r")

	SYH:ShowPanel()
end
